@@ -249,7 +249,8 @@ void init_cups(list_t *arglist, dstr_t *filelist, jobparams_t *job)
     char cups_user [128];
     char cups_jobtitle [128];
     char cups_copies [128];
-    char cups_options [512];
+    int cups_options_len;
+    char *cups_options;
     char cups_filename [256];
     char texttopspath[PATH_MAX];
 
@@ -270,7 +271,10 @@ void init_cups(list_t *arglist, dstr_t *filelist, jobparams_t *job)
     strncpy_omit(cups_user, arglist_get(arglist, 1), 128, omit_shellescapes);
     strncpy_omit(cups_jobtitle, arglist_get(arglist, 2), 128, omit_shellescapes);
     strncpy_omit(cups_copies, arglist_get(arglist, 3), 128, omit_shellescapes);
-    strncpy_omit(cups_options, arglist_get(arglist, 4), 512, omit_shellescapes);
+
+    cups_options_len = strlen(arglist_get(arglist, 4));
+    cups_options = malloc(cups_options_len + 1);
+    strncpy_omit(cups_options, arglist_get(arglist, 4), cups_options_len, omit_shellescapes);
 
     /* Common job parameters */
     strcpy(job->id, cups_jobid);
@@ -312,6 +316,8 @@ void init_cups(list_t *arglist, dstr_t *filelist, jobparams_t *job)
                     texttopspath, cups_jobid, cups_user, cups_jobtitle, cups_copies, cups_options);
         }
     }
+
+    free(cups_options);
 }
 
 void init_solaris(list_t *arglist, dstr_t *filelist, jobparams_t *job)
@@ -410,7 +416,7 @@ int configfile_find_option(const char *configfile, const char *key, char *dest,
         if (!prefixcmp(line, "default")) {
             p = strchr(line, ':');
             if (p) {
-                strncpy_omit(dest, p, destsize, omit_whitespace);
+                strncpy_omit(dest, p + 1, destsize, omit_whitespace_newline);
                 if (dest[0])
                     break;
             }
@@ -479,7 +485,7 @@ void init_direct_cps_pdq(list_t *arglist, dstr_t *filelist, jobparams_t *job)
 
         /* Search for the PPD file */
         if (!find_ppdfile(user_default_path, job)) {
-            _log("There is no readable PPD file for the printer %s, is it configured?\n");
+            _log("There is no readable PPD file for the printer %s, is it configured?\n", job->printer);
             exit(EXIT_PRNERR_NORETRY_BAD_SETTINGS);
         }
     }
